package com.gangstar;

import android.util.FloatMath;

import com.google.android.maps.GeoPoint;

public class DistanceCalculator {

	/**
	 * The radius of the earth (either kilometers
	 * or meters)
	 */
	private double radius;
	
	/**
	 * R = Earth's radius (mean radius = 6 371 km)
	 * Constructor
	 */
	DistanceCalculator(double R) {
		this.radius = R;
	}
	
	/**
	 * Find the distance between two places (in meters)
	 * @param lat_a the latitude of the start point
	 * @param lng_a the longititude of the start point
	 * @param lat_b the latitude of the end point
	 * @param lng_b the longititude of the end point
	 * @return the distance of the points in meters
	 */
	public double gps2m(float lat_a, float lng_a, float lat_b, float lng_b) { 
	    float pk = (float) (180/3.14169); 

	    float a1 = lat_a / pk; 
	    float a2 = lng_a / pk; 
	    float b1 = lat_b / pk; 
	    float b2 = lng_b / pk; 

	    float t1 = FloatMath.cos(a1)*FloatMath.cos(a2)*FloatMath.cos(b1)*FloatMath.cos(b2); 
	    float t2 = FloatMath.cos(a1)*FloatMath.sin(a2)*FloatMath.cos(b1)*FloatMath.sin(b2); 
	    float t3 = FloatMath.sin(a1)*FloatMath.sin(b1); 
	    double tt = Math.acos(t1 + t2 + t3); 
	    
	    return 6366000*tt; 
	}
	
	/**
	 * Calculates the distance between two GeoPoints
	 * @param startPoint
	 * @param endPoint
	 * @return
	 */
	public double CalculationByDistance(GeoPoint startPoint, GeoPoint endPoint) {
		double lat1 = startPoint.getLatitudeE6()/1E6;
		double lat2 = endPoint.getLatitudeE6()/1E6;
	    double lon1 = startPoint.getLongitudeE6()/1E6; 
	    double lon2 = endPoint.getLongitudeE6()/1E6; 
	    double dLat = Math.toRadians(lat2-lat1); 
	    double dLon = Math.toRadians(lon2-lon1); 
	    double a = Math.sin(dLat/2) * Math.sin(dLat/2) + 
	    Math.cos(Math.toRadians(lat1)) * Math.cos(Math.toRadians(lat2)) * 
	    Math.sin(dLon/2) * Math.sin(dLon/2); 
	    double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a)); 
	    return radius * c; 
	}
}
