/**
 * @author Japskua
 *	
 * The Gangstar Family interface, which promises all the
 * basic functionalities of a family. This is actually an
 * Observable of the Observer pattern, so it can provide the 
 * updates of the family to other gangsters easily.
 */

package com.gangstar;

public interface Family {

	/**
	 *
	 * This function is used to join the family in question
	 * @author Japskua
	 * @param takes the Gangster class gangster which is to be added 
	 * to be a part of the family
	 * @return true if successful, false if something went horribly wrong
	 */
	public void JoinFamily(Gangster gangster);
	/**
	 * This function is used to leave from the family (and also
	 * the unsubscribe from being an observer to that family)
	 * @author Japskua
	 * @param gangster the name of the Gangster class
	 */
	public void LeaveFamily(Gangster gangster);
	/**
	 * getState() is used to get the current state of the Family (when updates happen,
	 * for example when the family loses areas etc...)
	 * @author Japskua
	 * @return the current state of the Family as a string
	 */
	public String getState();
	
	/**
	 * This function is used to set the state of the family, when something changes
	 * (for example the family member loses or gains a territory)
	 * @author Japskua
	 * @param state the new state that is wanted to be announced to other 
	 * family members as a String
	 */
	public void setState(String state);
	
}
