package com.gangstar;

import java.util.ArrayList;

import android.widget.Toast;

/**
 * Gangster class holds all the information related
 * to a certain gangster, containing his name, the family
 * he/she belongs to, the amount of goons, what kind of weapons
 * they have, what areas he/she currently controls and how much
 * money they provide each day. Gangster also implements the Observer part
 * of the observer pattern, as it follows the current situation of the family
 * @author Japskua
 * @version 1.0
 *
 */
public class Gangster {
	
	private String gangsterName;
	private Family family;
	private int goons;
	private int money;
	private Weapon weapon;
	private ArrayList<Area>areas = new ArrayList<Area>();
	private String state;
	private int revenues;

	/**
	 * The creator for the Gangster takes the gangster's name
	 * and the family he/she belongs to as an input
	 * @author Japskua
	 * @param name The name of the gangster as a String
	 * @param family the Family which the gangster belongs to
	 */
	public Gangster(String name, Family family) {
		gangsterName = name;
		this.family = family;
		goons = 5;
		weapon = new WeaponPistol();
		money = 0;
		areas.clear();
	}
	
	/**
	 * @author Japskua
	 * @return the Family object of the gangster in question
	 */
	public Family getFamily() {
		return family;
	}
	
	/**
	 * @author Japskua
	 * @return the name of the gangster
	 */
	public String getName() {
		return gangsterName;
	}
	
	/**
	 * 
	 * @author Japskua
	 * @return the areas which the user controls as an ArrayList
	 */
	public ArrayList<Area> getAreas() {
		return areas;
	}
	
	/**
	 * 
	 * @author Japskua
	 * @param area the Area object of the area that is added to the gangster
	 */
	public void addArea(Area area) {
		areas.add(area);
		revenues = revenues + area.getRevenue();
	}
	
	/**
	 * @author Japskua
	 * @param area the Area object that is to be removed from the gangster
	 */
	public void removearea(Area area) {
		areas.remove(area);
		revenues = revenues - area.getRevenue();
	}
	
	public int getRevenues() {
		return revenues;
	}
	
	
	
	/**
	 * @author Japskua
	 * @return the amount of goons the player controls
	 */
	public int getGoons() {
		return goons;
	}
	
	/**
	 * Adds goons to the current Gangsters companions
	 * @param amount the amount of troops to be added
	 */
	public void addGoons(int amount) {
		goons = goons + amount;
	}
	
	/**
	 * Removes the amount of goons specified from the 
	 * Gangsters companions. This amount can go never below 
	 * zero.
	 * @param amount the amount of troops to be removed
	 */
	public void removeGoons(int amount) {
		goons = goons - amount;
		if (goons < 0) {
			goons = 0;
		}
	}
	
	/**
	 * @author Japskua
	 * @return the current weapon which the player is using
	 */
	public Weapon getWeapon() {
		return weapon;
	}
	
	/**
	 * Changes the gangster's current weapon to a new one
	 * @author Japskua
	 * @param weapon the Weapon that is used to replace the current one
	 */
	public void setWeapon(Weapon weapon) {
		this.weapon = weapon;
	}
	
	/**
	 * @author Japskua
	 * @return returns the amount of money the gangster has as 
	 * an integer
	 */
	public int getMoney() {
		return money;
	}
	
	/**
	 * This function is used to change the amount of money
	 * the gangster currently has
	 * @author Japskua
	 * @param the amount of money that is set as user's current available
	 * money.
	 */
	public void setMoney(int money) {
		this.money = money;
	}
	
	/**
	 * Gets the current state of the family
	 * @author Japskua
	 * @return the state of the family as a String
	 */
	public String getState() {
		return state;
	}
	
	/**
	 * Setstate is used to set the current known state to a new one
	 * @author Japskua
	 * @param state
	 */
	public void setState(String state) {
		this.state = state;
	}

	/**
	 * This function is used to update the current state of the family 
	 * (this is what the Observable calls all it's members to use when things
	 * have changed).
	 * @author Japskua
	 * @param state
	 */
	public void update(String state) {
		setState(state);
		// Tell the player that he has received a new state
		System.out.println("State changed to: " + getState());
		
	}



}


