package com.gangstar;

import com.google.android.maps.GeoPoint;

/**
 * PlayerSingleton holds the Players information, e.g. 
 * current location data in a private GeoPoint. This is a
 * Singleton which can be used globally by any class
 * @author Japskua
 * @version 1.0
 *
 */
public class PlayerSingleton {
	private static PlayerSingleton instance = null;
	private GeoPoint myLocation;
	private Gangster player;
	private boolean attackerWon = false;
	private int troopsLeft;
	
	protected PlayerSingleton() {
		// Only to prevent instantiation
	}
	/**
	 * @author Japskua
	 * @return the instance of the Location Singleton
	 */
	public static PlayerSingleton getInstance() {
		if (instance == null) {
			instance = new PlayerSingleton();
		}
		return instance;
	}
	
	/**
	 * @author Japskua
	 * @return the location of the current user as 
	 * a GeoPoint
	 */
	public GeoPoint getMyLocation() {
		return myLocation;
	}
	/**
	 * @author Japskua
	 * @param myPoint The GeoPoint that is set to be 
	 * the user's current position
	 */
	public void setMyLocation(GeoPoint myPoint) {
		this.myLocation = myPoint;
	}
	
	public void setPlayer(Gangster gangster) {
		this.player = gangster;
	}
	
	public Gangster getPlayer() {
		return player;
	}

	public void setAttackerWon() {
		attackerWon = true;
	}
	
	public boolean getAttackerWon() {
		return attackerWon;
	}
	
	public void setAttackerLost() {
		attackerWon = false;
	}
	
	/**
	 * Sets the amount of troops left on the winning side of the battle
	 * @author Japskua
	 * @param troopsLeft the amount of troops left as an integer
	 */
	public void setTroopsLeft(int troopsLeft) {
		this.troopsLeft = troopsLeft;
	}
	/**
	 * The amount of troops left AFTER THE FIGHT ON THE WINNING SIDE
	 * @author Japskua
	 * @return the amount of troops left on the winning side after the battle
	 */
	public int getTroopsLeft() {
		return troopsLeft;
	}
}
